@@ -4,7 +4,7 @@ use warnings;
 use 5.010; # state
 
 # ABSTRACT: turn on Unicode - all of it
-our $VERSION = '0.016'; # VERSION
+our $VERSION = '0.017'; # VERSION
 
 
 use Import::Into;
@@ -30,11 +30,15 @@ sub import {
 
         # Replace readdir with utf8 aware version
         *{$target . '::readdir'} = \&_utf8_readdir;
-        $^H{'utf8::all::readdir'} = 1; # Track whether to encode/decode in the redefined function
+
+        # Replace readdir with utf8 aware version
+        *{$target . '::readlink'} = \&_utf8_readlink;
 
         # Replace glob with utf8 aware version
         *{$target . '::glob'} = \&_utf8_glob;
-        $^H{'utf8::all::glob'} = 1;
+
+        # Set compiler hint to encode/decode in the redefined functions
+        $^H{'utf8::all'} = 1;
     }
 
     # Make @ARGV utf-8 when called from the main package, unless perl was launched
@@ -48,7 +52,20 @@ sub import {
         }
     }
 
-    $^H{'utf8::all'} = 1;
+    return;
+}
+
+sub unimport { ## no critic (Subroutines::ProhibitBuiltinHomonyms)
+    # Disable features/pragmas in calling package
+    # Note: Does NOT undo the effect on @ARGV,
+    #       nor on the STDIN, STDOUT, and STDERR file handles!
+    #       These effects are always "global".
+
+    my $target = caller;
+    'utf8'->unimport::out_of($target);
+    'open'->import::into($target, qw{IO :bytes});
+
+    $^H{'utf8::all'} = 0; # Reset compiler hint
 
     return;
 }
@@ -57,7 +74,7 @@ sub _utf8_readdir(*) { ## no critic (Subroutines::ProhibitSubroutinePrototypes)
     my $pre_handle = shift;
     my $handle = ref($pre_handle) ? $pre_handle : qualify_to_ref($pre_handle, caller);
     my $hints = (caller 0)[10];
-    if (not $hints->{'utf8::all::readdir'}) {
+    if (not $hints->{'utf8::all'}) {
         return CORE::readdir($handle);
     } elsif (wantarray) {
         return map { Encode::decode('UTF-8' ,$_) } CORE::readdir($handle);
@@ -66,10 +83,20 @@ sub _utf8_readdir(*) { ## no critic (Subroutines::ProhibitSubroutinePrototypes)
     }
 }
 
+sub _utf8_readlink(_) { ## no critic (Subroutines::ProhibitSubroutinePrototypes)
+    my $arg = shift;
+    my $hints = (caller 0)[10];
+    if (not $hints->{'utf8::all'}) {
+        return CORE::readlink($arg);
+    } else {
+        return Encode::decode('UTF-8', CORE::readlink(Encode::encode('UTF-8', $arg)));
+    }
+}
+
 sub _utf8_glob {
     my $arg = $_[0]; # Making this a lexical somehow is important!
     my $hints = (caller 0)[10];
-    if (not $hints->{'utf8::all::glob'}) {
+    if (not $hints->{'utf8::all'}) {
         return CORE::glob($arg);
     } else {
         $arg = Encode::encode('UTF-8', $arg);
@@ -96,11 +123,11 @@ utf8::all - turn on Unicode - all of it
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 SYNOPSIS
 
-    use utf8::all;                # Turn on UTF-8, all of it.
+    use utf8::all;                      # Turn on UTF-8, all of it.
 
     open my $in, '<', 'contains-utf8';  # UTF-8 already turned on here
     print length 'føø bār';             # 7 UTF-8 characters
@@ -108,8 +135,10 @@ version 0.016
 
 =head1 DESCRIPTION
 
-L<utf8> allows you to write your Perl encoded in UTF-8. That means
-UTF-8 strings, variable names, and regular expressions.
+The C<use utf8> pragma tells the Perl parser to allow UTF-8 in the
+program text in the current lexical scope. This also means that you
+can now use literal Unicode characters as part of strings, variable
+names, and regular expressions.
 
 C<utf8::all> goes further:
 
@@ -117,27 +146,39 @@ C<utf8::all> goes further:
 
 =item *
 
-Makes C<@ARGV> encoded in UTF-8 (when C<utf8::all> is used from the main package).
+L<C<charnames>|charnames> are imported so C<\N{...}> sequences can be
+used to compile Unicode characters based on names.
+
+=item *
+
+On Perl C<v5.11.0> or higher, the C<use feature 'unicode_strings'> is
+enabled.
+
+=item *
+
+C<use feature fc> and C<use feature unicode_eval> are enabled on Perl
+C<5.16.0> and higher.
 
 =item *
 
 Filehandles are opened with UTF-8 encoding turned on by default
-(including STDIN, STDOUT, STDERR). If you I<don't> want UTF-8 for a
-particular filehandle, you'll have to set C<binmode $filehandle>.
+(including STDIN, STDOUT, STDERR). Meaning that they automatically
+convert UTF-8 octets to characters and vice versa. If you I<don't>
+want UTF-8 for a particular filehandle, you'll have to set C<binmode
+$filehandle>.
 
 =item *
 
-L<charnames> are imported so C<\N{...}> sequences can be used to compile
-Unicode characters based on names.
+C<@ARGV> gets converted from UTF-8 octets to Unicode characters (when
+C<utf8::all> is used from the main package). This is similar to the
+behaviour of the C<-CA> perl command-line switch (see L<perlrun>).
 
 =item *
 
-readdir now returns UTF-8 characters instead of bytes.
-
-=item *
-
-L<glob|perlfunc/glob> and the C<< <> >> operator now return UTF-8
-characters instead of bytes.
+C<readdir>, C<readlink>, C<readpipe> (including the C<qx//> and
+backtick operators), and L<C<glob>|perlfunc/glob> (including the C<< <>
+>> operator) now all work with and return Unicode characters instead
+of (UTF-8) octets.
 
 =back
 
@@ -157,6 +198,12 @@ some reason to:
     my $text = do { local $/; <$in>};
     print length $text, "\n";         # 10, not 7!
 
+Instead of lexical scoping, you can also use C<no utf8::all> to turn
+off the effects.
+
+Note that the effect on C<@ARGV> and the C<STDIN>, C<STDOUT>, and
+C<STDERR> file handles is always global!
+
 =head1 SEE ALSO
 
 =over 4
@@ -200,8 +247,8 @@ web interface at L<https://github.com/doherty/utf8-all/issues>.
 =head1 COMPATIBILITY
 
 The filesystems of Dos, Windows, and OS/2 do not (fully) support
-UTF-8. The C<readdir> function and C<glob> operators will therefore not
-be replaced on these systems.
+UTF-8. The C<readlink> and C<readdir> functions and C<glob> operators
+will therefore not be replaced on these systems.
 
 =head1 AUTHORS
 
